@@ -1,84 +1,4 @@
-//! # css-inline
-//!
-//! A crate for inlining CSS into HTML documents. When you send HTML emails, you need to use "style"
-//! attributes instead of "style" tags.
-//!
-//! For example, this HTML:
-//!
-//! ```html
-//! <html>
-//!     <head>
-//!         <title>Test</title>
-//!         <style>h1 { color:blue; }</style>
-//!     </head>
-//!     <body>
-//!         <h1>Big Text</h1>
-//!     </body>
-//! </html>
-//! ```
-//!
-//! Will be turned into this:
-//!
-//! ```html
-//! <html>
-//!     <head><title>Test</title></head>
-//!     <body>
-//!         <h1 style="color:blue;">Big Text</h1>
-//!     </body>
-//! </html>
-//! ```
-//!
-//! ## Usage
-//!
-//! ```rust
-//! const HTML: &str = r#"<html>
-//! <head>
-//!     <title>Test</title>
-//!     <style>h1 { color:blue; }</style>
-//! </head>
-//! <body>
-//!     <h1>Big Text</h1>
-//! </body>
-//! </html>"#;
-//!
-//!fn main() -> Result<(), css_inline::InlineError> {
-//!    let inlined = css_inline::inline(HTML)?;  // shortcut with default options
-//!    // Do something with inlined HTML, e.g. send an email
-//!    Ok(())
-//! }
-//! ```
-//!
-//! ### Features & Configuration
-//!
-//! `css-inline` can be configured by using `CSSInliner::options()` that implements the Builder pattern:
-//!
-//! ```rust
-//! const HTML: &str = r#"<html>
-//! <head>
-//!     <title>Test</title>
-//!     <style>h1 { color:blue; }</style>
-//! </head>
-//! <body>
-//!     <h1>Big Text</h1>
-//! </body>
-//! </html>"#;
-//!
-//! fn main() -> Result<(), css_inline::InlineError> {
-//!     let inliner = css_inline::CSSInliner::options()
-//!         .load_remote_stylesheets(false)
-//!         .build();
-//!     let inlined = inliner.inline(HTML);
-//!     // Do something with inlined HTML, e.g. send an email
-//!     Ok(())
-//! }
-//! ```
-//!
-//! - `inline_style_tags`. Whether to inline CSS from "style" tags. Default: `true`
-//! - `remove_style_tags`. Remove "style" tags after inlining. Default: `false`
-//! - `base_url`. Base URL to resolve relative URLs. Default: `None`
-//! - `load_remote_stylesheets`. Whether remote stylesheets should be loaded or not. Default: `true`
-//! - `extra_css`. Additional CSS to inline. Default: `None`
-//!
+#![doc = include_str!("../README.md")]
 #![warn(
     clippy::doc_markdown,
     clippy::redundant_closure,
@@ -115,9 +35,29 @@ mod parser;
 use ahash::AHashMap;
 pub use error::InlineError;
 use smallvec::{smallvec, SmallVec};
-use std::{borrow::Cow, collections::hash_map::Entry, fs, io::Write};
+use std::{
+    borrow::Cow,
+    collections::hash_map::Entry,
+    fs,
+    io::{ErrorKind, Write},
+};
 pub use url::{ParseError, Url};
 
+/// Replace double quotes in property values.
+///
+/// This implementation is deliberately simplistic and covers only `font-family`, but escaping
+/// might be needed in other properties that accept strings.
+macro_rules! replace_double_quotes {
+    ($target:expr, $name:expr, $value:expr) => {
+        // Avoid allocation if there is no double quote in the input string
+        if $name.starts_with("font-family") && memchr::memchr(b'"', $value.as_bytes()).is_some() {
+            $target.push_str(&$value.replace('"', "\'"))
+        } else {
+            $target.push_str($value)
+        };
+    };
+}
+
 /// Configuration options for CSS inlining process.
 #[derive(Debug)]
 pub struct InlineOptions<'a> {
@@ -258,6 +198,14 @@ impl<'a> CSSInliner<'a> {
 
     /// Inline CSS styles from <style> tags to matching elements in the HTML tree and return a
     /// string.
+    ///
+    /// # Errors
+    ///
+    /// Inlining might fail for the following reasons:
+    ///   - Missing stylesheet file;
+    ///   - Remote stylesheet is not available;
+    ///   - IO errors;
+    ///   - Internal CSS selector parsing error;
     #[inline]
     pub fn inline(&self, html: &str) -> Result<String> {
         // Allocating the same amount of memory as the input HTML helps to avoid
@@ -270,6 +218,14 @@ impl<'a> CSSInliner<'a> {
 
     /// Inline CSS & write the result to a generic writer. Use it if you want to write
     /// the inlined document to a file.
+    ///
+    /// # Errors
+    ///
+    /// Inlining might fail for the following reasons:
+    ///   - Missing stylesheet file;
+    ///   - Remote stylesheet is not available;
+    ///   - IO errors;
+    ///   - Internal CSS selector parsing error;
     #[inline]
     pub fn inline_to<W: Write>(&self, html: &str, target: &mut W) -> Result<()> {
         let document = parse_html().one(html);
@@ -352,7 +308,7 @@ impl<'a> CSSInliner<'a> {
                     for (name, (_, value)) in styles {
                         final_styles.push_str(name.as_str());
                         final_styles.push(':');
-                        final_styles.push_str(value.as_str());
+                        replace_double_quotes!(final_styles, name, &value);
                         final_styles.push(';');
                     }
                     attributes.insert("style", final_styles);
@@ -389,7 +345,12 @@ fn load_external(location: &str) -> Result<String> {
         let response = request.send()?;
         Ok(response.text()?)
     } else {
-        fs::read_to_string(location).map_err(InlineError::IO)
+        fs::read_to_string(location).map_err(|error| match error.kind() {
+            ErrorKind::NotFound => InlineError::MissingStyleSheet {
+                path: location.to_string(),
+            },
+            _ => InlineError::IO(error),
+        })
     }
 }
 
@@ -445,12 +406,27 @@ impl Default for CSSInliner<'_> {
 }
 
 /// Shortcut for inlining CSS with default parameters.
+///
+/// # Errors
+///
+/// Inlining might fail for the following reasons:
+///   - Missing stylesheet file;
+///   - Remote stylesheet is not available;
+///   - IO errors;
+///   - Internal CSS selector parsing error;
 #[inline]
 pub fn inline(html: &str) -> Result<String> {
     CSSInliner::default().inline(html)
 }
 
 /// Shortcut for inlining CSS with default parameters and writing the output to a generic writer.
+/// # Errors
+///
+/// Inlining might fail for the following reasons:
+///   - Missing stylesheet file;
+///   - Remote stylesheet is not available;
+///   - IO errors;
+///   - Internal CSS selector parsing error;
 #[inline]
 pub fn inline_to<W: Write>(html: &str, target: &mut W) -> Result<()> {
     CSSInliner::default().inline_to(html, target)
@@ -472,16 +448,16 @@ fn merge_styles(
         let (name, value) = declaration?;
         final_styles.push_str(&name);
         final_styles.push(':');
-        final_styles.push_str(value);
+        replace_double_quotes!(final_styles, name, value);
         final_styles.push(';');
         // This property won't be taken from new styles
-        buffer.push(name.to_string())
+        buffer.push(name.to_string());
     }
     for (property, (_, value)) in new_styles {
         if !buffer.contains(property) {
             final_styles.push_str(property);
             final_styles.push(':');
-            final_styles.push_str(value);
+            replace_double_quotes!(final_styles, property, value);
             final_styles.push(';');
         }
     }
