@@ -1,6 +1,6 @@
 #[macro_use]
 mod utils;
-use css_inline::{inline, CSSInliner, InlineOptions, Url};
+use css_inline::{inline, CSSInliner, InlineError, InlineOptions, Url};
 
 #[test]
 fn no_existing_style() {
@@ -68,6 +68,17 @@ fn overloaded_styles() {
     )
 }
 
+#[test]
+fn font_family_quoted() {
+    // When property value contains double quotes
+    assert_inlined!(
+        style = r#"h1 { font-family: "Open Sans", sans-serif; }"#,
+        body = r#"<h1>Hello world!</h1>"#,
+        // Then it should be replaced with single quotes
+        expected = r#"<h1 style="font-family: 'Open Sans', sans-serif;">Hello world!</h1>"#
+    )
+}
+
 #[test]
 fn existing_styles() {
     // When there is a `style` attribute on a tag that contains a rule
@@ -298,6 +309,25 @@ h2 { color: red; }
     ))
 }
 
+#[test]
+fn missing_stylesheet() {
+    let html = r#"
+<html>
+<head>
+<link href="tests/missing.css" rel="stylesheet" type="text/css">
+</head>
+<body>
+<h1>Big Text</h1>
+</body>
+</html>"#;
+    match inline(html).expect_err("Should be an error") {
+        InlineError::MissingStyleSheet { path } => {
+            assert_eq!(path, String::from("tests/missing.css"))
+        }
+        _ => panic!("Invalid error type"),
+    }
+}
+
 #[test]
 fn remote_file_stylesheet_disable() {
     let html = r#"
